(function (global) {
    "use strict";

    global.ND = global.ND || {};
    global.ND.Portal = global.ND.Portal || {};
    global.ND.Portal.FileNet = (function() {

      var downloadDocument = function(documentId, mimeType, baseUrl, selectorPrefix) {

        $(`#${selectorPrefix}Loading`).show();

        var data = JSON.stringify({
            "DocumentId": documentId,
            "InFileNet": true
        });

        $.ajax({
            url: `${baseUrl}/api/download?documentid=${documentId}&infilenet=true`,
            type: 'GET',
            contentType: 'application/json',
            dataType: 'json', 
            headers: {"Access-Control-Allow-Origin": "*"},
            success: function(response) {
              $(`#${selectorPrefix}Loading`).hide();

                if (response.File && response.FileName) {
                    
                    var blob = new Blob([convertBase64ToByteArray(response.File)], { type: mimeType });
                    var downloadUrl = URL.createObjectURL(blob);

                    executeDownloadLink(downloadUrl, response.FileName);
                }
            },
            error: function(xhr, status, error) {
              $(`#${selectorPrefix}Loading`).hide();
                console.error('Request failed. Status:', status, 'Error:', error);
            }
        });
      }

      var downloadDocumentV2 = function(documentId, mimeType, baseUrl, selectorPrefix) {
        return new Promise((resolve, reject) => {
            $(`#${selectorPrefix}Loading`).show();
    
            var data = JSON.stringify({
                "DocumentId": documentId,
                "InFileNet": true
            });
    
            $.ajax({
                url: `${baseUrl}/api/download?documentid=${documentId}&infilenet=true`,
                type: 'GET',
                contentType: 'application/json',
                dataType: 'json', 
                headers: {"Access-Control-Allow-Origin": "*"},
                success: function(response) {
                    $(`#${selectorPrefix}Loading`).hide();
    
                    if (response.File && response.FileName) {
                        var blob = new Blob([convertBase64ToByteArray(response.File)], { type: mimeType });
                        var downloadUrl = URL.createObjectURL(blob);
    
                        executeDownloadLink(downloadUrl, response.FileName);
                        return resolve(true);
                    } else {
                        return resolve(false);
                    }
                },
                error: function(xhr, status, error) {
                    $(`#${selectorPrefix}Loading`).hide();
                    console.error('Request failed. Status:', status, 'Error:', error);
                    return resolve(false);
                }
            });
        });
      }

      var convertBase64ToByteArray = function(base64String) {
        var binary = atob(base64String);
        var len = binary.length;
        var buffer = new ArrayBuffer(len);
        var view = new Uint8Array(buffer);
        for (var i = 0; i < len; i++) {
            view[i] = binary.charCodeAt(i);
        }

        return view;
      };

      var executeDownloadLink = function(downloadUrl, fileName) {
        var a = $('<a>')
        .attr('href', downloadUrl)
        .attr('target', '_blank')
        .attr('download', fileName)
        .appendTo('body');
        a[0].click(); // Simulate a click on the element to trigger the download
        a.remove(); // Remove the link from the document
        URL.revokeObjectURL(downloadUrl); // Release the object URL
      };
        
      return {
        downloadDocument: downloadDocument,
        downloadDocumentV2: downloadDocumentV2,
        convertBase64ToByteArray: convertBase64ToByteArray,
        executeDownloadLink: executeDownloadLink
      }
	})();

})(this);